gene2pathway.signaltrans = function(geneIDs=NULL, flyBase=FALSE, gene2Domains=NULL, organism="hsa", useKEGG=TRUE, mc.cores=8){	
	if(is.null(geneIDs) & is.null(gene2Domains))
		stop("You have to provide either a list of Entrez gene IDs or a mapping of genes to InterPro domains")	
	if(!exists("gene2pathwayEnv"))
		assign("gene2pathwayEnv",new.env(parent=globalenv()),envir=.GlobalEnv)
	if(exists("organismSignalTrans", envir=gene2pathwayEnv))
		old.organism = get("organismSignalTrans", envir=gene2pathwayEnv)
	else
		old.organism = organism
	assign("organismSignalTrans", organism, envir=gene2pathwayEnv)
	if(!is.null(gene2Domains))	
		geneIDs = names(gene2Domains)
	else
		geneIDs = as.character(geneIDs) 
	if(!exists("modelSignalTrans", envir=gene2pathwayEnv) | (organism != old.organism)){
		cat("Loading classification model ...\n")
		myfile = paste("classificationModelSignalTrans_",organism,sep="")
		tryCatch(data(list=myfile,package="gene2pathway", envir=gene2pathwayEnv), warning=function(w) stop("No Model for organism '", organism, "' available.\nPlease invoke 'retrain.signaltrans' to generate one."))
# 		data(list=myfile,package="gene2pathway", envir=gene2pathwayEnv)
# 		load(paste("classificationModelSignalTrans_",organism,".rda",sep=""), envir=gene2pathwayEnv)
	}
	model = get("modelSignalTrans", envir=gene2pathwayEnv)		
	if(class(model) == "model"){
		alldomains = model$alldomains
		elemIDs = model$elemIDs
		pathways = model$allpathways				
		kegg_hierarchy = model$kegg_hierarchy
	}
	else{
		alldomains = model[[1]]$alldomains		
		elemIDs = model[[1]]$elemIDs
		pathways = model[[1]]$allpathways		
		kegg_hierarchy = model[[1]]$kegg_hierarchy		
	}
	roots = unique(names(kegg_hierarchy$parentPaths))
    roots = intersect(roots, pathways)		
	
	if(useKEGG){			
		cat("Using KEGG information from SOAP service ...\n")			
		organisms=list.organisms()
		## organisms = sapply(organisms, function(x) x@entry_id)
		if(!(organism %in% organisms))
			stop(paste("Organism '", organism, "' unknown in KEGG! Please refer to <URL:http://www.genome.jp/kegg-bin/create_kegg_menu> for a complete list of supported organisms."))
		cat("Mapping to signal transduction pathway components via KEGG database ...\n",sep="")
		all.pathways = list.pathways(organism)
		pathways.names = all.pathways ##sapply(all.pathways, function(x) x@entry_id)
		all.pathways = names(all.pathways) ##sapply(all.pathways, function(x) x@definition)
		names(all.pathways) = pathways.names
		all.pathways = gsub(" - .*", "", all.pathways)	
		roots = roots[roots %in% all.pathways]
		pathwayIDs = all.pathways[all.pathways %in% roots]		
# 		if(!(organism %in% c("hsa", "mmu", "rno"))){
		if(flyBase){
			geneIDs = unlist(AnnotationDbi::mget(geneIDs, org.Dm.egFLYBASE2EG, ifnotfound=NA))
			geneIDs = geneIDs[!is.na(geneIDs)]
			if(length(geneIDs) == 0)
				stop("No mapping FlyBase -> Entrez gene ID found!")
			flyBase = FALSE				
		}
		if(organism != "hsa"){
#			KEGG2Entrez.tab = gene2pathway:::KEGG2Entrez(organism=organism)
#			geneIDs.conv = gene2pathway:::Entrez2ORF.internal(geneIDs, KEGG2Entrez.tab, organism=organism)	
			geneIDs.conv = try(bconv(paste("ncbi-geneid:", geneIDs, sep="")))
		}
		else{
			geneIDs.conv = geneIDs			
		}
		if ("doMC" %in% loadedNamespaces()){
			registerDoMC(mc.cores)
			comp2gene = foreach(p = 1:length(pathwayIDs)) %dopar%{
				elems = gene2pathway:::get.elements.by.pathway(names(pathwayIDs[p]))
				which.gene = sapply(elems, function(e) e@type == "gene")
				elems = elems[which.gene]
				compGenes = sapply(elemIDs[[pathwayIDs[p]]], function(c) unlist(sapply(elems[c], function(e) e@NAMES)))
				#sapply(compGenes, function(g) sub(paste(organism,":",sep=""),"", g))
			}
		}
		else{
			comp2gene = sapply(1:length(pathwayIDs), function(p){				
				elems = gene2pathway:::get.elements.by.pathway(names(pathwayIDs[p]))
				which.gene = sapply(elems, function(e) e@type == "gene")
				elems = elems[which.gene]
				compGenes = sapply(elemIDs[[pathwayIDs[p]]], function(c) unlist(sapply(elems[c], function(e) e@NAMES)))
				#sapply(compGenes, function(g) sub(paste(organism,":",sep=""),"", g))
			})
		}
		names(comp2gene) = roots			
		anno.genes = geneIDs.conv[geneIDs.conv %in% unique(unlist(comp2gene))]
		cat("---> Information found for ", length(anno.genes), "genes\n")
		KEGGgenes = lapply(anno.genes, function(kg){
			unlist(sapply(1:length(comp2gene), function(i){
				comp = sapply(comp2gene[[i]], function(cc) any(cc %in% kg))
				if(any(comp)){
					compname = intersect(paste(roots[i],which(comp),sep="."), pathways)
					if(length(compname) > 0)
						return(c(roots[i], compname))
					else
						return(roots[i])
				}				
			}))				
		})		
		names(KEGGgenes) = anno.genes
		if(length(grep(":", names(KEGGgenes))) > 0){
			if(organism == "hsa")						
				anno.genes = sub(paste(organism,":",sep=""), "", names(KEGGgenes))
			else{
				cat("converting KEGG gene identifiers to Entrez ...")
				genes.conv = try(bconv(names(KEGGgenes)))
				anno.genes = sub("ncbi-geneid:", "", genes.conv[grep("ncbi-geneid", genes.conv)])  #gene2pathway:::KEGG2Entrez(names(KEGGgenes), geneID.list=KEGG2Entrez.tab, organism=organism)				
			}
			names(KEGGgenes) = anno.genes				
		}
		cat("done.\n")		
		if(!is.null(gene2Domains)  && length(intersect(names(KEGGgenes), geneIDs)) == 0)
			warning("There may be a conflict between KEGG Entrez gene / open reading frame IDs and the gene identifiers in list 'gene2Domains'.\n Gene identifiers in 'gene2Domains' should be Entrez gene IDs.")
	}
	else
		KEGGgenes = list()		
	topredict = setdiff(geneIDs, names(KEGGgenes))
	if(length(topredict) > 0){		
		cat(length(topredict), " genes to predict\n")		
		features = gene2pathway:::getInterProDomains(topredict, gene2Domains=gene2Domains, alldoms=alldomains, organism=organism, flyBase=flyBase)	
		
		cat("Model prediction possible for ", nrow(features), " genes ")
		gene2Pathall = gene2pathway:::predict.gene2pathway(model, features, mc.cores=mc.cores)
		cat("done\nPreparing output\n")
		gene2Path = gene2Pathall$gene2Path	
		scorestmp = gene2Pathall$scores
		names(scorestmp) = rownames(features)
		votes = gene2Pathall$votes
	}
	else{
		scorestmp = NA
		gene2Path = list()
		votes = NA
	}
	
	totallist = vector("list", length=length(geneIDs))	
	scores = vector("list", length=length(geneIDs))	
	components = vector("list", length=length(geneIDs))
	totalvotes = vector("list", length=length(geneIDs))	
	names(totalvotes) = geneIDs
	names(totallist) = geneIDs
	names(scores) = geneIDs
	names(components) = geneIDs	
	totallist[names(gene2Path)] = gene2Path	
	scores[names(gene2Path)] = scorestmp	
	totalvotes[names(gene2Path)] = votes
	byKEGG = rep(FALSE, length(totallist))
	names(byKEGG) = names(totallist)
	if(useKEGG){
		totallist[names(KEGGgenes)] = KEGGgenes
		byKEGG[names(KEGGgenes)] = TRUE
	}	
	components[names(gene2Path)] = sapply(gene2Path, function(gp){			
		 lapply(gp[grep("\\.", gp)], function(gpp){			
			p = unlist(strsplit(gpp, "\\."))
			unlist(elemIDs[[p[1]]][p[2]])			
		})
	})	
# 	if(exists("kegg_hierarchy", envir=gene2pathwayEnv))
# 		kegg_hierarchy = get("kegg_hierarchy", envir=gene2pathwayEnv)
# 	else{
# 		kegg_hierarchy = gene2pathway:::getKEGGHierarchy(level1Only=c(), level2Only=c())
# 		assign("kegg_hierarchy", kegg_hierarchy, envir=gene2pathwayEnv)		
# 	}
#	pathnames = c(kegg_hierarchy$pathNamesLev1, kegg_hierarchy$pathNamesLev2, kegg_hierarchy$pathNamesLev3)		
#	totallist[totallist == ""] = NA
#	totallist[is.na(names(totallist))] = NA
#	gene2Pathname = sapply(totallist, function(gp) setdiff(pathnames[as.character(gp)], NA))	
#	names(gene2Pathname) = names(totallist)
	
	cat("finished\n")
	list(gene2Path=totallist, scores=scores, byKEGG=byKEGG, elemIDs=components, votes=totalvotes)
}
