predict.gene2pathway = function(object, newdata=NULL, diagnostics=FALSE, PDF=FALSE, mc.cores=8, ...){
# 1. predict codes
# 2. use codes to predict structures
	testall = function(fit, xtst){	
		cat(".")
		codes = gene2pathway:::code_test(fit$detectors, xtst)	
		# predict at all hierarchy levels
		gene2pathway:::struct_predict(fit, codes, diagnostics=diagnostics, PDF=PDF)		
	}
	
# prediction method for bagged model
	testallBag = function(model, xtst){	
		if ("doMC" %in% loadedNamespaces()){			
			pred = foreach(m = model) %dopar% {
				testall(m, xtst)$yhat[,,drop=FALSE]	
			}
		}
		else{
			pred = lapply(model, function(m) testall(m, xtst)$yhat[,,drop=FALSE])
		}
		test = pred[[1]]
		for(b in 2:length(model)){
			test = test + pred[[b]]
		}		
		votes = test/length(model)		
		test = (votes > 0.5)*1
		scores = apply(votes[,,drop=FALSE],1,function(v) 0.5*(mean(v[v > 0.5], na.rm=T) + 1-mean(v[v<=0.5], na.rm=T)))		
		list(yhat=test, scores=scores, votes=votes)	
	}
	
	if ("doMC" %in% loadedNamespaces())
		registerDoMC(mc.cores)
		
	if(missing(newdata))
		return(NULL)	
	if(class(object) == "model"){
		pred = testall(object, newdata)	
		scores = pred$diff	
		cnames = colnames(object$C)
		votes = NULL
	}
	else{
		pred =  testallBag(object, newdata)	
		scores = pred$scores
		cnames = colnames(object[[1]]$C)
		votes = pred$votes		
		dimnames(votes) = list(rownames(newdata), cnames)
		votes.matrix = votes
	}
	predcode = pred$yhat[,,drop=FALSE]
	dimnames(predcode) = list(rownames(newdata), cnames)	
	
# 	bereite Ergebnis in lesbarer Form auf
	gene2Path = lapply(1:nrow(predcode), function(p) colnames(predcode)[predcode[p,,drop=FALSE] == 1])	
	names(gene2Path) = rownames(predcode)	
	if(!is.null(votes)){			
		votes = lapply(1:nrow(votes), function(v) votes[v,gene2Path[[v]]])
		names(votes) = names(gene2Path)
	}
	list(gene2Path=gene2Path, scores=scores, votes=votes, votes.matrix=votes.matrix)
}
